// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#ifndef DUNE_CURVED_SURFACE_GRID_ENTITYSEED_HH
#define DUNE_CURVED_SURFACE_GRID_ENTITYSEED_HH

#include <dune/common/typetraits.hh>

#include <dune/curvedsurfacegrid/capabilities.hh>
#include <dune/grid/common/entityseed.hh>

namespace Dune
{
  namespace CGeo
  {

    // EntitySeed
    // ----------

    template< int codim, class G >
    class EntitySeed
    {
      using Traits = typename std::remove_const_t<G>::Traits;
      using HostGrid = typename Traits::HostGrid;
      using HostEntitySeed = typename HostGrid::template Codim<codim>::EntitySeed;

    public:
      static const int codimension = codim;

      //! default construct an invalid entity seed
      EntitySeed () = default;

      //! construct the entity-seed from a host-entity seed
      explicit EntitySeed (const HostEntitySeed& hostEntitySeed)
        : hostEntitySeed_(hostEntitySeed)
      {}

      //! check whether the EntitySeed refers to a valid Entity
      bool isValid () const
      {
        return hostEntitySeed_.isValid();
      }

      //! return the entity seed of the host-entity
      const HostEntitySeed& hostEntitySeed () const
      {
        return hostEntitySeed_;
      }

    private:
      HostEntitySeed hostEntitySeed_ = {};
    };

  } // namespace CGeo
} // namespace Dune

#endif // DUNE_CURVED_SURFACE_GRID_ENTITYSEED_HH
