// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#ifndef DUNE_CURVED_SURFACE_GRID_GRID_ENTITYSET_HH
#define DUNE_CURVED_SURFACE_GRID_GRID_ENTITYSET_HH

namespace Dune
{
  //! A set of entities of given `codim` of a `Grid`
  /**
   * \tparam G      The grid type
   * \tparam codim  Codimension of the entities to define the set of.
   *
   * \note This entityset just defines types
   **/
  template< class G, int codim >
  class GridEntitySet
  {
  public:
    //! Type of the grid
    using Grid = G;

    //! Type of Elements contained in this EntitySet
    using Element = typename Grid::template Codim<codim>::Entity;

    //! Type of local coordinates with respect to the Element
    using LocalCoordinate = typename Element::Geometry::LocalCoordinate;

    //! Type of global coordinates with respect to the Element
    using GlobalCoordinate = typename Element::Geometry::GlobalCoordinate;
  };

} // end namespace Dune 

#endif // DUNE_CURVED_SURFACE_GRID_GRID_ENTITYSET_HH
