// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#ifndef DUNE_CURVED_SURFACE_GRID_INTERSECTIONITERATOR_HH
#define DUNE_CURVED_SURFACE_GRID_INTERSECTIONITERATOR_HH

#include <dune/curvedsurfacegrid/intersection.hh>

namespace Dune
{
  namespace CGeo
  {

    // IntersectionIterator
    // --------------------

    template< class Grid, class HostIntersectionIterator >
    class IntersectionIterator
    {
      using IntersectionImpl = CGeo::Intersection<Grid, typename HostIntersectionIterator::Intersection>;

      using Traits = typename std::remove_const_t<Grid>::Traits;
      using GridFunction = typename Traits::GridFunction;
      using Element = typename Traits::template Codim<0>::Entity;
      using ElementGeometry = typename Traits::template Codim<0>::Geometry;
      //using ElementGeometryImpl = typename Traits::template Codim<0>::GeometryImpl;

    public:
      using Intersection = Dune::Intersection< Grid, IntersectionImpl>;

      IntersectionIterator() = default;

      //template< class Entity >
      IntersectionIterator (const Element& inside, const HostIntersectionIterator& hostIterator)
        : hostIterator_(hostIterator)
        , gridFunction_(&inside.impl().gridFunction())
      {}

      bool equals (const IntersectionIterator& other) const
      {
        return (hostIterator_ == other.hostIterator_);
      }

      void increment ()
      {
        ++hostIterator_;
      }

      Intersection dereference () const
      {
        return IntersectionImpl(*hostIterator_, *gridFunction_);
      }

    private:
      HostIntersectionIterator hostIterator_;
      const GridFunction* gridFunction_ = nullptr;
    };

  } // namespace CGeo
} // namespace Dune

#endif // DUNE_CURVED_SURFACE_GRID_INTERSECTIONITERATOR_HH
